@@ -22,6 +22,7 @@ import os
 import sys
 
 from cliff import app
+from cliff import command
 from cliff import help
 
 import openstackclient
@@ -29,6 +30,7 @@ from openstackclient.common import clientmanager
 from openstackclient.common import commandmanager
 from openstackclient.common import exceptions as exc
 from openstackclient.common import openstackkeyring
+from openstackclient.common import restapi
 from openstackclient.common import utils
 
 
@@ -37,6 +39,7 @@ KEYRING_SERVICE = 'openstack'
 DEFAULT_COMPUTE_API_VERSION = '2'
 DEFAULT_IDENTITY_API_VERSION = '2.0'
 DEFAULT_IMAGE_API_VERSION = '1'
+DEFAULT_OBJECT_API_VERSION = '1'
 DEFAULT_VOLUME_API_VERSION = '1'
 DEFAULT_DOMAIN = 'default'
 
@@ -62,6 +65,11 @@ class OpenStackShell(app.App):
     log = logging.getLogger(__name__)
 
     def __init__(self):
+        # Patch command.Command to add a default auth_required = True
+        command.Command.auth_required = True
+        # But not help
+        help.HelpCommand.auth_required = False
+
         super(OpenStackShell, self).__init__(
             description=__doc__.strip(),
             version=openstackclient.__version__,
@@ -186,6 +194,15 @@ class OpenStackShell(app.App):
             help='Image API version, default=' +
                  DEFAULT_IMAGE_API_VERSION +
                  ' (Env: OS_IMAGE_API_VERSION)')
+        parser.add_argument(
+            '--os-object-api-version',
+            metavar='<object-api-version>',
+            default=env(
+                'OS_OBJECT_API_VERSION',
+                default=DEFAULT_OBJECT_API_VERSION),
+            help='Object API version, default=' +
+                 DEFAULT_OBJECT_API_VERSION +
+                 ' (Env: OS_OBJECT_API_VERSION)')
         parser.add_argument(
             '--os-volume-api-version',
             metavar='<volume-api-version>',
@@ -338,14 +355,15 @@ class OpenStackShell(app.App):
             'compute': self.options.os_compute_api_version,
             'identity': self.options.os_identity_api_version,
             'image': self.options.os_image_api_version,
+            'object-store': self.options.os_object_api_version,
             'volume': self.options.os_volume_api_version,
         }
 
         # Add the API version-specific commands
         for api in self.api_version.keys():
             version = '.v' + self.api_version[api].replace('.', '_')
-            self.command_manager.add_command_group(
-                'openstack.' + api + version)
+            cmd_group = 'openstack.' + api.replace('-', '_') + version
+            self.command_manager.add_command_group(cmd_group)
 
         # Commands that span multiple APIs
         self.command_manager.add_command_group(
@@ -368,19 +386,16 @@ class OpenStackShell(app.App):
         if self.options.deferred_help:
             self.DeferredHelpAction(self.parser, self.parser, None, None)
 
-        # If the user is not asking for help, make sure they
-        # have given us auth.
-        cmd_name = None
-        if argv:
-            cmd_info = self.command_manager.find_command(argv)
-            cmd_factory, cmd_name, sub_argv = cmd_info
-        if self.interactive_mode or cmd_name != 'help':
-            self.authenticate_user()
+        # Set up common client session
+        self.restapi = restapi.RESTApi()
 
     def prepare_to_run_command(self, cmd):
         """Set up auth and API versions"""
         self.log.debug('prepare_to_run_command %s', cmd.__class__.__name__)
-        self.log.debug("api: %s" % cmd.api if hasattr(cmd, 'api') else None)
+
+        if cmd.auth_required:
+            self.authenticate_user()
+            self.restapi.set_auth(self.client_manager.identity.auth_token)
         return
 
     def clean_up(self, cmd, result, err):
@@ -388,6 +403,13 @@ class OpenStackShell(app.App):
         if err:
             self.log.debug('got an error: %s', err)
 
+    def interact(self):
+        # NOTE(dtroyer): Maintain the old behaviour for interactive use as
+        #                this path does not call prepare_to_run_command()
+        self.authenticate_user()
+        self.restapi.set_auth(self.client_manager.identity.auth_token)
+        super(OpenStackShell, self).interact()
+
 
 def main(argv=sys.argv[1:]):
     try:
